
#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Heikin-Ashi Candlestick Oscillator as described in the December 2008 issue of S&C.
    /// </summary>
    [Description("Heikin-Ashi Candlestick Oscillator as described in the December 2008 issue of S&C.")]
    public class HACO : Indicator
    {
        #region Variables
		private int avg = 34; // Default setting for Avg
		private int avgDn = 34; // Default setting for AvgDn
		
		private DataSeries haOpen;
		private DataSeries haC;
		private DataSeries tMA1;
		private DataSeries tMA2;
		private DataSeries tMA3;
		private DataSeries tMA4;
		private DataSeries tMA5;
		private DataSeries tMA6;
		private DataSeries tMA7;
		private DataSeries tMA8;
		
		private double zlHa;
		private double zlCl;
		private double zlDif;
		private double diff;
		
		private bool keep1 = false;
		private bool keep2 = false;
		private bool keep3 = false;
		private BoolSeries keeping;
		private BoolSeries keepall;
		private BoolSeries keeping2;
		private BoolSeries keepall2;
		
		private BoolSeries utr;
		private BoolSeries dtr;
		private bool upw = false;
		private bool dnw = false;
		private BoolSeries result;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Add(new Plot(new Pen(Color.Black, 2), "HACO"));
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
			
			haOpen 	= new DataSeries(this);
			haC 	= new DataSeries(this);
			tMA1 	= new DataSeries(this);
			tMA2 	= new DataSeries(this);
			tMA3 	= new DataSeries(this);
			tMA4 	= new DataSeries(this);
			tMA5 	= new DataSeries(this);
			tMA6 	= new DataSeries(this);
			tMA7 	= new DataSeries(this);
			tMA8 	= new DataSeries(this);
			
			keeping 	= new BoolSeries(this);
			keepall 	= new BoolSeries(this);
			keeping2 	= new BoolSeries(this);
			keepall2 	= new BoolSeries(this);
			utr 		= new BoolSeries(this);
			dtr 		= new BoolSeries(this);
			result 		= new BoolSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < 2)
				return;
			
			// UPTREND
			haOpen.Set((((Open[1] + High[1] + Low[1] + Close[1]) / 4) + haOpen[1]) / 2);
			haC.Set(((Open[0] + High[0] + Low[0] + Close[0]) / 4 + haOpen[0] + Math.Max(High[0], haOpen[0]) + Math.Min(Low[0], haOpen[0])) / 4);
			
			tMA1.Set(TEMA(haC, Avg)[0]);
			tMA2.Set(TEMA(tMA1, Avg)[0]);
			diff = tMA1[0] - tMA2[0];
			zlHa = tMA1[0] + diff;
			
			tMA3.Set(TEMA(Median, Avg)[0]);
			tMA4.Set(TEMA(tMA3, Avg)[0]);
			diff 	= tMA3[0] - tMA4[0];
			zlCl 	= tMA3[0] + diff;
			zlDif 	= zlCl - zlHa;
			
			// Keep Bools			
			if (MRO(delegate {return haC[0] >= haOpen[0];}, 1, 2) > -1)
				keep1 = true;
			else
				keep1 = false;
			
			keep2 = (zlDif >= 0) ? true : false;
			
			if (keep1 || keep2)
				keeping.Set(true);
			else
				keeping.Set(false);
			
			if (keeping[0] || (keeping[1] && (Close[0] >= 0) || Close[0] >= Close[1]))
				keepall.Set(true);
			else
				keepall.Set(false);
			
			keep3 = ((Math.Abs(Close[0] - Open[0]) < (High[0] - Low[0]) * 0.35) && (High[0] >= Low[1])) ? true : false;
			
			if (keepall[0] || (keepall[1] && keep3))
				utr.Set(true);
			else
				utr.Set(false);
			
			// DOWNTREND
			tMA5.Set(TEMA(haC, AvgDn)[0]);
			tMA6.Set(TEMA(tMA5, AvgDn)[0]);
			diff = tMA5[0] - tMA6[0];
			zlHa = tMA5[0] + diff;
			
			tMA7.Set(TEMA(Median, AvgDn)[0]);
			tMA8.Set(TEMA(tMA7, AvgDn)[0]);
			diff 	= tMA7[0] - tMA8[0];
			zlCl 	= tMA7[0] + diff;			
			zlDif 	= zlCl - zlHa;
			
			// Keep Bools			
			if (MRO(delegate {return haC[0] < haOpen[0];}, 1, 2) > -1)
				keep1 = true;
			else
				keep1 = false;
			
			keep2 = (zlDif < 0) ? true : false;
			keep3 = ((Math.Abs(Close[0] - Open[0]) < (High[0] - Low[0]) * 0.35) && (Low[0] <= High[1])) ? true : false;
			
			if (keep1 || keep2)
				keeping2.Set(true);
			else
				keeping2.Set(false);
			
			if (keeping2[0] || (keeping2[1] && (Close[0] < Open[0]) || Close[0] < Close[1]))
				keepall2.Set(true);
			else
				keepall2.Set(false);
			
			
			// Output Routine
			if (keepall2[0] || (keepall2[1] && keep3))
				dtr.Set(true);
			else
				dtr.Set(false);
			
			if (dtr[0] == false && dtr[1] && utr[0])
				upw = true;
			else
				upw = false;
			
			if (utr[0] == false && utr[1] && dtr[0])
				dnw = true;
			else
				dnw = false;
			
			if (upw)
			{
				Result.Set(1);
				BackColorAll = Color.MediumSeaGreen;
			}
			else if (dnw)
			{
				Result.Set(0);
				BackColorAll = Color.Pink;
			}
			else if (dnw == false)
			{
				Result.Set(Result[1]);
				if (Result[0] == 1)
					BackColorAll = Color.MediumSeaGreen;
				else
					BackColorAll = Color.Pink;
			}
        }

        #region Properties
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Result
		{
			get { return Values[0]; }
		}
		
        [Description("Up TEMA average")]
        [Category("Parameters")]
        public int Avg
        {
            get { return avg; }
            set { avg = Math.Max(1, value); }
        }

        [Description("Down TEMA average")]
        [Category("Parameters")]
        public int AvgDn
        {
            get { return avgDn; }
            set { avgDn = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private HACO[] cacheHACO = null;

        private static HACO checkHACO = new HACO();

        /// <summary>
        /// Heikin-Ashi Candlestick Oscillator as described in the December 2008 issue of S&C.
        /// </summary>
        /// <returns></returns>
        public HACO HACO(int avg, int avgDn)
        {
            return HACO(Input, avg, avgDn);
        }

        /// <summary>
        /// Heikin-Ashi Candlestick Oscillator as described in the December 2008 issue of S&C.
        /// </summary>
        /// <returns></returns>
        public HACO HACO(Data.IDataSeries input, int avg, int avgDn)
        {
            checkHACO.Avg = avg;
            avg = checkHACO.Avg;
            checkHACO.AvgDn = avgDn;
            avgDn = checkHACO.AvgDn;

            if (cacheHACO != null)
                for (int idx = 0; idx < cacheHACO.Length; idx++)
                    if (cacheHACO[idx].Avg == avg && cacheHACO[idx].AvgDn == avgDn && cacheHACO[idx].EqualsInput(input))
                        return cacheHACO[idx];

            HACO indicator = new HACO();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Avg = avg;
            indicator.AvgDn = avgDn;
            indicator.SetUp();

            HACO[] tmp = new HACO[cacheHACO == null ? 1 : cacheHACO.Length + 1];
            if (cacheHACO != null)
                cacheHACO.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheHACO = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Heikin-Ashi Candlestick Oscillator as described in the December 2008 issue of S&C.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.HACO HACO(int avg, int avgDn)
        {
            return _indicator.HACO(Input, avg, avgDn);
        }

        /// <summary>
        /// Heikin-Ashi Candlestick Oscillator as described in the December 2008 issue of S&C.
        /// </summary>
        /// <returns></returns>
        public Indicator.HACO HACO(Data.IDataSeries input, int avg, int avgDn)
        {
            return _indicator.HACO(input, avg, avgDn);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Heikin-Ashi Candlestick Oscillator as described in the December 2008 issue of S&C.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.HACO HACO(int avg, int avgDn)
        {
            return _indicator.HACO(Input, avg, avgDn);
        }

        /// <summary>
        /// Heikin-Ashi Candlestick Oscillator as described in the December 2008 issue of S&C.
        /// </summary>
        /// <returns></returns>
        public Indicator.HACO HACO(Data.IDataSeries input, int avg, int avgDn)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.HACO(input, avg, avgDn);
        }

    }
}
#endregion
